from dataclasses import dataclass
from enum import Enum
from gmo_fx.api.api_base import PublicApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Symbol
from requests import get, Response
from gmo_fx.urls import BASE_URL_PUBLIC


@dataclass
class Rule:
    Symbol = Symbol
    symbol: Symbol
    min_open_order_size: int
    max_order_size: int
    size_step: int
    tick_size: float


class SymbolsResponse(ResponseBase):
    rules: list[Rule]

    def __init__(self, response: dict):
        super().__init__(response)
        self.rules = []

        data = response["data"]
        self.rules = [
            Rule(
                symbol=Rule.Symbol(d["symbol"]),
                min_open_order_size=int(d["minOpenOrderSize"]),
                max_order_size=int(d["maxOrderSize"]),
                size_step=int(d["sizeStep"]),
                tick_size=float(d["tickSize"]),
            )
            for d in data
        ]


class SymbolsApi(PublicApiBase):

    @property
    def _path(self) -> str:
        return f"symbols"

    @property
    def _method(self) -> PublicApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return SymbolsResponse

    def _api_error_message(self, response: Response):
        return (
            "取引ルールが取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )



import pytest
from gmo_fx.errors import ApiError
from gmo_fx.api.response import Response
from datetime import datetime


class TestBaseResponse:
    convert_response_testdata = [
        (0, datetime(2001, 12, 12)),
        (0, datetime(2010, 3, 1)),
        (0, datetime(2020, 5, 20)),
    ]

    @pytest.mark.parametrize("status, time", convert_response_testdata)
    def test_convert_response(
        self,
        status: int,
        time: datetime,
    ):
        data = {
            "status": status,
            "responsetime": time.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
        }
        response = Response(data)
        assert response.response_status == status
        assert response.response_time == time

    error_codes = [
        143,
        148,
        189,
        200,
        201,
        254,
        414,
        423,
        512,
        542,
        595,
        635,
        759,
        760,
        761,
        769,
        786,
        5003,
        5008,
        5009,
        5010,
        5011,
        5012,
        5014,
        5106,
        5114,
        5122,
        5123,
        5125,
        5126,
        5130,
        5132,
        5201,
        5202,
        5204,
        5206,
        5207,
        5208,
        5209,
        5210,
        5211,
        5213,
        5214,
        5218,
        5219,
        5220,
        5221,
        5222,
        5223,
        5224,
        5225,
        5226,
        5227,
        5228,
        5229,
    ]

    def create_error_data(self, error_code: str, message: str) -> dict:
        return {
            "status": 1,
            "messages": [
                {
                    "message_code": error_code,
                    "message_string": message,
                }
            ],
            "responsetime": datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
        }

    def test_should_return_error_status_is_not_0(self) -> None:
        with pytest.raises(ApiError) as e:
            Response(
                {
                    "status": 1,
                    "messages": [
                        {
                            "message_code": "ERR-5011",
                            "message_string": "API-key format invalid.",
                        }
                    ],
                    "responsetime": datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
                }
            )

    @pytest.mark.parametrize("error_code", error_codes)
    def test_error_code_test(self, error_code) -> None:
        with pytest.raises(ApiError) as e:
            data = self.create_error_data(
                f"ERR-{error_code}", "API-key format invalid."
            )
            Response(data)
        assert e.value.error_code == f"ERR-{error_code}"



from enum import Enum


class Symbol(Enum):
    """銘柄"""

    USD_JPY = "USD_JPY"
    EUR_JPY = "EUR_JPY"
    GBP_JPY = "GBP_JPY"
    AUD_JPY = "AUD_JPY"
    NZD_JPY = "NZD_JPY"
    CAD_JPY = "CAD_JPY"
    CHF_JPY = "CHF_JPY"
    TRY_JPY = "TRY_JPY"
    ZAR_JPY = "ZAR_JPY"
    MXN_JPY = "MXN_JPY"
    EUR_USD = "EUR_USD"
    GBP_USD = "GBP_USD"
    AUD_USD = "AUD_USD"
    NZD_USD = "NZD_USD"


class Side(Enum):
    """売買区分
    BUY: 売
    SELL: 買
    """

    BUY = "BUY"
    SELL = "SELL"


class SettleType(Enum):
    """決済区分"""

    OPEN = "OPEN"
    CLOSE = "CLOSE"


class OrderType(Enum):
    """取引区分"""

    NORMAL = "NORMAL"
    OCO = "OCO"



from datetime import datetime, timezone
from tests.api_test_base import ApiTestBase
from typing import Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.executions import (
    Execution,
    ExecutionsApi,
    ExecutionsResponse,
)


class TestExecutionsApi(ApiTestBase):

    def call_api(
        self,
        order_id: Optional[list[int]] = None,
        execution_id: Optional[list[int]] = None,
    ) -> ExecutionsResponse:
        return ExecutionsApi(
            api_key="",
            secret_key="",
        )(
            order_id=order_id,
            execution_id=execution_id,
        )

    def create_response(
        self,
        data: Optional[dict] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        if data is None:
            data = {"list": [self.create_execution_data()]}

        return super().create_response(
            data=data,
            status_code=status_code,
            text=text,
        )

    def create_execution_data(
        self,
        execution_id: int = 92123912,
        client_order_id: Optional[str] = "abc123",
        order_id: int = 223456789,
        position_id: int = 2234567,
        symbol: str = "USD_JPY",
        side: str = "BUY",
        settle_type: str = "OPEN",
        size: int = 10000,
        price: float = 141.269,
        loss_gain: float = 0,
        fee: float = 0,
        settled_swap: float = 0,
        amount: float = 0,
        timestamp: str = "2020-11-24T19:47:51.234Z",
    ) -> dict:
        data = {
            "executionId": execution_id,
            "orderId": order_id,
            "positionId": position_id,
            "symbol": symbol,
            "side": side,
            "settleType": settle_type,
            "size": str(size),
            "price": str(price),
            "lossGain": str(loss_gain),
            "fee": str(fee),
            "settledSwap": str(settled_swap),
            "amount": str(amount),
            "timestamp": timestamp,
        }

        if client_order_id is not None:
            data["clientOrderId"] = client_order_id

        return data

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api(execution_id=[123]))

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(execution_id=[123])
        url: str = get_mock.mock_calls[0].args[0]
        assert url.startswith("https://forex-api.coin.z.com/private/v1/executions")

    def check_parse_a_data(self, get_mock: MagicMock, **kwargs) -> ExecutionsResponse:
        get_mock.return_value = self.create_response(
            data={"list": [self.create_execution_data(**kwargs)]}
        )
        response = self.call_api(execution_id=[123])
        assert len(response.executions) == 1
        return response

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_execution_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, execution_id=2536464541)
        assert response.executions[0].execution_id == 2536464541

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_client_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, client_order_id="abbb324dff")
        assert response.executions[0].client_order_id == "abbb324dff"

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_none_without_client_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, client_order_id=None)
        assert response.executions[0].client_order_id is None

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, order_id=156789)
        assert response.executions[0].order_id == 156789

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, position_id=9876543)
        assert response.executions[0].position_id == 9876543

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbol(self, get_mock: MagicMock):
        for symbol in Execution.Symbol:
            response = self.check_parse_a_data(get_mock, symbol=symbol.value)
            assert response.executions[0].symbol == symbol

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_side(self, get_mock: MagicMock):
        for side in Execution.Side:
            response = self.check_parse_a_data(get_mock, side=side.value)
            assert response.executions[0].side == side

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settle_type(self, get_mock: MagicMock):
        for settle_type in Execution.SettleType:
            response = self.check_parse_a_data(get_mock, settle_type=settle_type.value)
            assert response.executions[0].settle_type == settle_type

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_size(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, size=6100)
        assert response.executions[0].size == 6100

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_price(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, price=155.44)
        assert response.executions[0].price == 155.44

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_loss_gain(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, loss_gain=15730)
        assert response.executions[0].loss_gain == 15730

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_fee(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, fee=-30)
        assert response.executions[0].fee == -30

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settled_swap(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, settled_swap=515.999)
        assert response.executions[0].settled_swap == 515.999

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_amount(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, amount=16215.999)
        assert response.executions[0].amount == 16215.999

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_timestamp(self, get_mock: MagicMock):
        response = self.check_parse_a_data(
            get_mock, timestamp="2024-09-13T15:21:03.059Z"
        )
        assert response.executions[0].timestamp == datetime(
            2024, 9, 13, 15, 21, 3, 59000, tzinfo=timezone.utc
        )

    @patch("gmo_fx.api.api_base.get")
    def test_should_parse_multiple_data(
        self, get_mock: MagicMock
    ) -> ExecutionsResponse:
        get_mock.return_value = self.create_response(
            data={"list": [self.create_execution_data(), self.create_execution_data()]}
        )
        response = self.call_api(execution_id=[123])
        assert len(response.executions) == 2

    def check_call_with(self, get_mock, **kwargs):
        get_mock.return_value = self.create_response()
        self.call_api(**kwargs)
        kall = get_mock.call_args
        url = kall.args[0] if kall.args else ""
        if "?" in url:
            query_string = url.split("?", 1)[1]
            params = {}
            for param in query_string.split("&"):
                if "=" in param:
                    key, value = param.split("=", 1)
                    params[key] = value
            return params
        return {}

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_single_execution_id(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, execution_id=[72123911])
        assert params["executionId"] == "72123911"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_multiple_execution_ids(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, execution_id=[72123911, 92123912])
        assert params["executionId"] == "72123911,92123912"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_single_order_id(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, order_id=[123456789])
        assert params["orderId"] == "123456789"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_multiple_order_ids(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, order_id=[123456789, 987654321])
        assert params["orderId"] == "123456789,987654321"

    @patch("gmo_fx.api.api_base.get")
    def test_should_parse_empty_list(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"list": []})
        response = self.call_api(execution_id=[123])
        assert len(response.executions) == 0



from dataclasses import dataclass
from datetime import datetime, timezone
from typing import Optional
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Side, Symbol, SettleType


@dataclass
class Execution:
    Side = Side
    Symbol = Symbol
    SettleType = SettleType

    execution_id: int
    order_id: int
    position_id: int
    symbol: Symbol
    side: Side
    settle_type: SettleType
    size: int
    price: float
    loss_gain: float
    fee: float
    settled_swap: float
    amount: float
    timestamp: datetime
    client_order_id: Optional[str] = None


class ExecutionsResponse(ResponseBase):
    executions: list[Execution]

    def __init__(self, response: dict):
        super().__init__(response)

        data = response["data"]["list"]
        self.executions = [
            Execution(
                execution_id=int(d["executionId"]),
                order_id=int(d["orderId"]),
                position_id=int(d["positionId"]),
                symbol=Execution.Symbol(d["symbol"]),
                side=Execution.Side(d["side"]),
                settle_type=Execution.SettleType(d["settleType"]),
                size=int(d["size"]),
                price=float(d["price"]),
                loss_gain=float(d["lossGain"]),
                fee=float(d["fee"]),
                settled_swap=float(d["settledSwap"]),
                amount=float(d["amount"]),
                timestamp=datetime.strptime(
                    d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"
                ).replace(tzinfo=timezone.utc),
                client_order_id=d.get("clientOrderId"),
            )
            for d in data
        ]


class ExecutionsApi(PrivateApiBase):
    @property
    def _path(self) -> str:
        return "executions"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return ExecutionsResponse

    def _api_error_message(self, response: Response):
        return (
            "約定情報が取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        order_id: list[int] = None,
        execution_id: list[int] = None,
    ) -> ExecutionsResponse:
        query_params = []
        if execution_id:
            query_params.append(
                f"executionId={','.join([str(id) for id in execution_id])}"
            )
        if order_id:
            query_params.append(f"orderId={','.join([str(id) for id in order_id])}")

        path_query = "&".join(query_params) if query_params else None
        return super().__call__(path_query=path_query)



from dataclasses import dataclass
from datetime import datetime, timezone
from enum import Enum
from typing import Optional
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Side, Symbol


@dataclass
class OpenPosition:
    Side = Side
    Symbol = Symbol

    position_id: int
    symbol: Symbol
    side: Side
    size: int
    ordered_size: int
    price: float
    loss_gain: float
    total_swap: float
    timestamp: datetime


class OpenPositionsResponse(ResponseBase):
    open_positions: list[OpenPosition]

    def __init__(self, response: dict):
        super().__init__(response)
        self.open_positions = []

        data = response["data"]["list"]
        self.open_positions = [
            OpenPosition(
                position_id=d["positionId"],
                symbol=OpenPosition.Symbol(d["symbol"]),
                side=OpenPosition.Side(d["side"]),
                size=int(d["size"]),
                ordered_size=int(d["orderedSize"]),
                price=float(d["price"]),
                loss_gain=float(d["lossGain"]),
                total_swap=float(d["totalSwap"]),
                timestamp=datetime.strptime(
                    d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"
                ).replace(tzinfo=timezone.utc),
            )
            for d in data
        ]


class OpenPositionsApi(PrivateApiBase):
    Symbol = Symbol

    @property
    def _path(self) -> str:
        return "openPositions"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return OpenPositionsResponse

    def _api_error_message(self, response: Response):
        return (
            "建玉一覧が取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        symbol: Optional[Symbol] = None,
        prev_id: Optional[int] = None,
        count: Optional[int] = None,
    ) -> OpenPositionsResponse:
        path_query = ""
        if symbol:
            path_query = f"symbol={symbol.value}"

        if prev_id:
            if path_query:
                path_query += "&"
            path_query = f"prevId={prev_id}"

        if count:
            if path_query:
                path_query += "&"
            path_query = f"count={count}"

        return super().__call__(path_query=path_query)



from dataclasses import dataclass
from datetime import date, datetime, timezone
from enum import Enum
from typing import Optional
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Side, Symbol, SettleType


@dataclass
class Order:
    class OrderType(Enum):
        NORMAL = "NORMAL"
        OCO = "OCO"
        IFD = "IFD"
        IFDOCO = "IFDOCO"
        LOSSCUT = "LOSSCUT"

    class ExecutionType(Enum):
        MARKET = "MARKET"
        LIMIT = "LIMIT"
        STOP = "STOP"

    class Status(Enum):
        WAITING = "WAITING"
        ORDERED = "ORDERED"
        MODIFYING = "MODIFYING"
        CANCELED = "CANCELED"
        EXECUTED = "EXECUTED"
        EXPIRED = "EXPIRED"

    class CancelType(Enum):
        USER = "USER"
        INSUFFICIENT_COLLATERAL = "INSUFFICIENT_COLLATERAL"
        INSUFFICIENT_MARGIN = "INSUFFICIENT_MARGIN"
        SPEED = "SPEED"
        OCO = "OCO"
        EXPIRATION = "EXPIRATION"
        PRICE_BOUND = "PRICE_BOUND"
        OUT_OF_SLIPPAGE_RANGE = "OUT_OF_SLIPPAGE_RANGE"

    Side = Side
    Symbol = Symbol
    SettleType = SettleType

    root_order_id: int
    order_id: int
    symbol: Symbol
    side: Side
    order_type: OrderType
    execution_type: ExecutionType
    settle_type: SettleType
    size: int
    price: Optional[float]
    status: Status
    expiry: Optional[date] = None
    timestamp: datetime = None
    client_order_id: Optional[str] = None
    cancel_type: Optional[CancelType] = None


class OrdersResponse(ResponseBase):
    orders: list[Order]

    def __init__(self, response: dict):
        super().__init__(response)

        data = response["data"]["list"]
        self.orders = [
            Order(
                root_order_id=int(d["rootOrderId"]),
                order_id=int(d["orderId"]),
                symbol=Order.Symbol(d["symbol"]),
                side=Order.Side(d["side"]),
                order_type=Order.OrderType(d["orderType"]),
                execution_type=Order.ExecutionType(d["executionType"]),
                settle_type=Order.SettleType(d["settleType"]),
                size=int(d["size"]),
                price=float(d["price"]) if d.get("price") else None,
                status=Order.Status(d["status"]),
                expiry=(
                    datetime.strptime(d["expiry"], "%Y%m%d").date()
                    if d.get("expiry")
                    else None
                ),
                timestamp=datetime.strptime(
                    d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"
                ).replace(tzinfo=timezone.utc),
                client_order_id=d.get("clientOrderId"),
                cancel_type=(
                    Order.CancelType(d["cancelType"]) if d.get("cancelType") else None
                ),
            )
            for d in data
        ]


class OrdersApi(PrivateApiBase):
    @property
    def _path(self) -> str:
        return "orders"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return OrdersResponse

    def _api_error_message(self, response: Response):
        return (
            "注文情報が取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        root_order_id: list[int] = [],
        order_id: list[int] = [],
    ) -> OrdersResponse:
        query_params = []
        if root_order_id:
            query_params.append(
                f"rootOrderId={','.join([str(id) for id in root_order_id])}"
            )
        if order_id:
            query_params.append(f"orderId={','.join([str(id) for id in order_id])}")

        path_query = "&".join(query_params) if query_params else None
        return super().__call__(path_query=path_query)



from dataclasses import dataclass
from enum import Enum
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Side, Symbol


@dataclass
class Position:
    Side = Side
    Symbol = Symbol

    average_position_rate: float
    position_loss_gain: float
    side: Side
    sum_ordered_size: int
    sum_position_size: int
    sum_total_swap: float
    symbol: Symbol


class PositionSummaryResponse(ResponseBase):
    positions: list[Position]

    def __init__(self, response: dict):
        super().__init__(response)
        self.assets = []

        data = response["data"]["list"]
        self.positions = [
            Position(
                average_position_rate=float(d["averagePositionRate"]),
                position_loss_gain=float(d["positionLossGain"]),
                side=Position.Side(d["side"]),
                sum_ordered_size=int(d["sumOrderedSize"]),
                sum_position_size=int(d["sumPositionSize"]),
                sum_total_swap=float(d["sumTotalSwap"]),
                symbol=Position.Symbol(d["symbol"]),
            )
            for d in data
        ]


class PositionSummaryApi(PrivateApiBase):
    Symbol = Symbol

    @property
    def _path(self) -> str:
        return "positionSummary"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return PositionSummaryResponse

    def _api_error_message(self, response: Response):
        return (
            "建玉サマリーが取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        symbol: Symbol,
    ) -> PositionSummaryResponse:
        return super().__call__(path_query=f"symbol={symbol.value}")



from typing import Callable, Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.ticker import Ticker, TickerApi, TickerResponse
from datetime import datetime

from tests.api_test_base import ApiTestBase


class TestTickerApi(ApiTestBase):
    SYMBOLS = [
        "USD_JPY",
        "EUR_JPY",
        "GBP_JPY",
        "AUD_JPY",
        "NZD_JPY",
        "CAD_JPY",
        "CHF_JPY",
        "TRY_JPY",
        "ZAR_JPY",
        "MXN_JPY",
        "EUR_USD",
        "GBP_USD",
        "AUD_USD",
        "NZD_USD",
    ]

    SYMBOLS_TABLE = {
        Ticker.Symbol.USD_JPY: "USD_JPY",
        Ticker.Symbol.EUR_JPY: "EUR_JPY",
        Ticker.Symbol.GBP_JPY: "GBP_JPY",
        Ticker.Symbol.AUD_JPY: "AUD_JPY",
        Ticker.Symbol.NZD_JPY: "NZD_JPY",
        Ticker.Symbol.CAD_JPY: "CAD_JPY",
        Ticker.Symbol.CHF_JPY: "CHF_JPY",
        Ticker.Symbol.TRY_JPY: "TRY_JPY",
        Ticker.Symbol.ZAR_JPY: "ZAR_JPY",
        Ticker.Symbol.MXN_JPY: "MXN_JPY",
        Ticker.Symbol.EUR_USD: "EUR_USD",
        Ticker.Symbol.GBP_USD: "GBP_USD",
        Ticker.Symbol.AUD_USD: "AUD_USD",
        Ticker.Symbol.NZD_USD: "NZD_USD",
    }

    def create_ticker_data(
        self,
        symbol: str,
        status: str = "OPEN",
        ask: float = 0.0,
        bid: float = 0.0,
        timestamp: Optional[datetime] = None,
    ) -> dict:
        return {
            "symbol": symbol,
            "ask": ask,
            "bid": bid,
            "timestamp": (timestamp or datetime.now()).strftime(
                "%Y-%m-%dT%H:%M:%S.%fZ"
            ),
            "status": status,
        }

    def create_tickers_data(
        self, exchange_data_builder: Optional[Callable[[str], dict]] = None
    ) -> list[dict]:
        exchange_data_builder = exchange_data_builder or self.create_ticker_data
        return [exchange_data_builder(symbol) for symbol in self.SYMBOLS]

    def call_api(
        self,
    ) -> TickerResponse:
        return TickerApi()()

    @patch("gmo_fx.api.api_base.get")
    def test_ticker_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbols_from_ticker(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data=self.create_tickers_data())
        ticker_response = self.call_api()
        symbols = [ticker.symbol for ticker in ticker_response.tickers]
        for symbol in Ticker.Symbol:
            assert symbol in symbols
            symbols.remove(symbol)
        else:
            assert len(symbols) == 0

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_bid_ask_from_ticker(self, get_mock: MagicMock):
        def fixture_ask_bid(symbol: str):
            return self.SYMBOLS.index(symbol), self.SYMBOLS.index(symbol) * 100

        def fixture_ticker_data(symbol: str) -> dict:
            ask, bid = fixture_ask_bid(symbol)
            return self.create_ticker_data(symbol, ask=ask, bid=bid)

        get_mock.return_value = self.create_response(
            data=self.create_tickers_data(exchange_data_builder=fixture_ticker_data)
        )
        rates = self.call_api().tickers
        for rate in rates:
            assert rate.ask == fixture_ask_bid(self.SYMBOLS_TABLE[rate.symbol])[0]
            assert rate.bid == fixture_ask_bid(self.SYMBOLS_TABLE[rate.symbol])[1]

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_timestamp_from_ticker(self, get_mock: MagicMock):
        def fixture_timestamp(symbol: str):
            return datetime(2000, 12, self.SYMBOLS.index(symbol) + 1)  # 0日はないので

        def fixture_ticker_data(symbol: str) -> dict:
            timestamp = fixture_timestamp(symbol)
            return self.create_ticker_data(symbol, timestamp=timestamp)

        get_mock.return_value = self.create_response(
            data=self.create_tickers_data(exchange_data_builder=fixture_ticker_data)
        )
        rates = self.call_api().tickers
        for rate in rates:
            assert rate.timestamp == fixture_timestamp(self.SYMBOLS_TABLE[rate.symbol])

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_status_from_ticker(self, get_mock: MagicMock):
        def fixture_status(symbol: str):
            statuses = ["OPEN", "CLOSE"]
            return statuses[self.SYMBOLS.index(symbol) % len(statuses)]

        def fixture_ticker_data(symbol: str) -> dict:
            status = fixture_status(symbol)
            return self.create_ticker_data(symbol, status=status)

        get_mock.return_value = self.create_response(
            data=self.create_tickers_data(exchange_data_builder=fixture_ticker_data)
        )
        rates = self.call_api().tickers
        for rate in rates:
            status_str = fixture_status(self.SYMBOLS_TABLE[rate.symbol])
            for status in ["OPEN", "CLOSE"]:
                if status == status_str:
                    assert rate.status == status
                    break
            else:
                assert False

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=self.create_tickers_data())
        self.call_api()
        url = get_mock.mock_calls[0].args[0]
        assert url == "https://forex-api.coin.z.com/public/v1/ticker"



from dataclasses import dataclass
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase
from gmo_fx.api.response import Response as ResponseBase


@dataclass
class Asset:
    equity: int
    available_amount: int
    balance: int
    estimated_trade_fee: float
    margin: int
    margin_ratio: float
    position_loss_gain: float
    total_swap: float
    transferable_amount: int


class AssetsResponse(ResponseBase):
    assets: list[Asset]

    def __init__(self, response: dict):
        super().__init__(response)
        self.assets = []

        data = response["data"]
        self.assets = [
            Asset(
                equity=d["equity"],
                available_amount=d["availableAmount"],
                balance=d["balance"],
                estimated_trade_fee=d["estimatedTradeFee"],
                margin=d["margin"],
                margin_ratio=d["marginRatio"],
                position_loss_gain=d["positionLossGain"],
                total_swap=d["totalSwap"],
                transferable_amount=d["transferableAmount"],
            )
            for d in data
        ]


class AssetsApi(PrivateApiBase):

    @property
    def _path(self) -> str:
        return "account/assets"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return AssetsResponse

    def _api_error_message(self, response: Response):
        return (
            "資産残高が取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )



from typing import Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.position_summary import (
    Position,
    PositionSummaryApi,
    PositionSummaryResponse,
)

from tests.api_test_base import ApiTestBase


class TestPositionSummaryApi(ApiTestBase):

    def create_response(
        self,
        data: Optional[list[dict]] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        return super().create_response(
            data={"list": data},
            status_code=status_code,
            text=text,
        )

    def call_api(
        self,
        symbol: PositionSummaryApi.Symbol = PositionSummaryApi.Symbol.USD_JPY,
    ) -> PositionSummaryResponse:
        return PositionSummaryApi(
            api_key="",
            secret_key="",
        )(symbol=symbol)

    def create_position_summary_data(
        self,
        average_position_rate: float = 0.0,
        position_loss_gain: float = 0.0,
        side: str = "BUY",
        sum_ordered_size: int = 0,
        sum_position_size: int = 0,
        sum_total_swap: float = 0.0,
        symbol: str = "USD_JPY",
    ) -> dict:
        return {
            "averagePositionRate": str(average_position_rate),
            "positionLossGain": str(position_loss_gain),
            "side": side,
            "sumOrderedSize": str(sum_ordered_size),
            "sumPositionSize": str(sum_position_size),
            "sumTotalSwap": str(sum_total_swap),
            "symbol": symbol,
        }

    def check_parse_a_data(self, get_mock: MagicMock, **kwargs) -> Position:
        get_mock.return_value = self.create_response(
            data=[self.create_position_summary_data(**kwargs)]
        )
        response = self.call_api()
        assert len(response.positions) == 1
        return response.positions[0]

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_average_position_rate(self, get_mock: MagicMock):
        position = self.check_parse_a_data(get_mock=get_mock, average_position_rate=1.1)
        assert position.average_position_rate == 1.1

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_loss_gain(self, get_mock: MagicMock):
        position = self.check_parse_a_data(get_mock=get_mock, position_loss_gain=23.54)
        assert position.position_loss_gain == 23.54

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_side(self, get_mock: MagicMock):
        for side in Position.Side:

            position = self.check_parse_a_data(get_mock=get_mock, side=side.value)
            assert position.side == side

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_sum_ordered_size(self, get_mock: MagicMock):
        position = self.check_parse_a_data(get_mock=get_mock, sum_ordered_size=984)
        assert position.sum_ordered_size == 984

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_sum_position_size(self, get_mock: MagicMock):
        position = self.check_parse_a_data(get_mock=get_mock, sum_position_size=1984)
        assert position.sum_position_size == 1984

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_sum_total_swap(self, get_mock: MagicMock):
        position = self.check_parse_a_data(get_mock=get_mock, sum_total_swap=654.6)
        assert position.sum_total_swap == 654.6

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_symbol(self, get_mock: MagicMock):
        for symbol in Position.Symbol:
            position = self.check_parse_a_data(get_mock=get_mock, symbol=symbol.value)
            assert position.symbol == symbol

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_some_positions(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=[
                self.create_position_summary_data(symbol="USD_JPY"),
                self.create_position_summary_data(symbol="GBP_USD"),
            ]
        )
        response = self.call_api()
        symbols = [position.symbol for position in response.positions]
        assert symbols[0] == Position.Symbol.USD_JPY
        assert symbols[1] == Position.Symbol.GBP_USD

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=[])
        self.call_api()
        url = get_mock.mock_calls[0].args[0]
        assert (
            url
            == "https://forex-api.coin.z.com/private/v1/positionSummary?symbol=USD_JPY"
        )

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_symbol(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=[])
        self.call_api(symbol=PositionSummaryApi.Symbol.AUD_JPY)
        url = get_mock.mock_calls[0].args[0]
        assert "symbol=AUD_JPY" in url



from datetime import datetime, timezone
from tests.api_test_base import ApiTestBase
from typing import Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.orders import (
    Order,
    OrdersApi,
    OrdersResponse,
)


class TestOrdersApi(ApiTestBase):

    def call_api(
        self,
        root_order_id: Optional[list[int]] = None,
        order_id: Optional[list[int]] = None,
    ) -> OrdersResponse:
        return OrdersApi(
            api_key="",
            secret_key="",
        )(
            root_order_id=root_order_id,
            order_id=order_id,
        )

    def create_response(
        self,
        data: Optional[dict] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        if data is None:
            data = {"list": [self.create_order_data()]}

        return super().create_response(
            data=data,
            status_code=status_code,
            text=text,
        )

    def create_order_data(
        self,
        root_order_id: int = 123456789,
        client_order_id: Optional[str] = "abc123",
        order_id: int = 123456789,
        symbol: str = "USD_JPY",
        side: str = "BUY",
        order_type: str = "NORMAL",
        execution_type: str = "LIMIT",
        settle_type: str = "OPEN",
        size: int = 10000,
        price: Optional[float] = 135.5,
        status: str = "ORDERED",
        expiry: Optional[str] = "20190418",
        timestamp: str = "2019-03-19T01:07:24.217Z",
        cancel_type: Optional[str] = None,
    ) -> dict:
        data = {
            "rootOrderId": root_order_id,
            "orderId": order_id,
            "symbol": symbol,
            "side": side,
            "orderType": order_type,
            "executionType": execution_type,
            "settleType": settle_type,
            "size": str(size),
            "status": status,
            "timestamp": timestamp,
        }

        if client_order_id is not None:
            data["clientOrderId"] = client_order_id

        if price is not None:
            data["price"] = str(price)

        if cancel_type is not None:
            data["cancelType"] = cancel_type

        if expiry is not None:
            data["expiry"] = expiry

        return data

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api()
        url: str = get_mock.mock_calls[0].args[0]
        assert url.startswith("https://forex-api.coin.z.com/private/v1/orders")

    def check_parse_a_data(self, get_mock: MagicMock, **kwargs) -> OrdersResponse:
        get_mock.return_value = self.create_response(
            data={"list": [self.create_order_data(**kwargs)]}
        )
        response = self.call_api()
        assert len(response.orders) == 1
        return response

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_root_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, root_order_id=2536464541)
        assert response.orders[0].root_order_id == 2536464541

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_client_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, client_order_id="abbb324dff")
        assert response.orders[0].client_order_id == "abbb324dff"

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_none_without_client_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, client_order_id=None)
        assert response.orders[0].client_order_id is None

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_order_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, order_id=156789)
        assert response.orders[0].order_id == 156789

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbol(self, get_mock: MagicMock):
        for symbol in Order.Symbol:
            response = self.check_parse_a_data(get_mock, symbol=symbol.value)
            assert response.orders[0].symbol == symbol

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_side(self, get_mock: MagicMock):
        for side in Order.Side:
            response = self.check_parse_a_data(get_mock, side=side.value)
            assert response.orders[0].side == side

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_order_type(self, get_mock: MagicMock):
        for order_type in Order.OrderType:
            response = self.check_parse_a_data(get_mock, order_type=order_type.value)
            assert response.orders[0].order_type == order_type

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_execution_type(self, get_mock: MagicMock):
        for execution_type in Order.ExecutionType:
            response = self.check_parse_a_data(
                get_mock, execution_type=execution_type.value
            )
            assert response.orders[0].execution_type == execution_type

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settle_type(self, get_mock: MagicMock):
        for settle_type in Order.SettleType:
            response = self.check_parse_a_data(get_mock, settle_type=settle_type.value)
            assert response.orders[0].settle_type == settle_type

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_size(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, size=6100)
        assert response.orders[0].size == 6100

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_price(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, price=155.44)
        assert response.orders[0].price == 155.44

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_none_without_price(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, price=None)
        assert response.orders[0].price is None

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_status(self, get_mock: MagicMock):
        for status in Order.Status:
            response = self.check_parse_a_data(get_mock, status=status.value)
            assert response.orders[0].status == status

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_cancel_type(self, get_mock: MagicMock):
        for cancel_type in Order.CancelType:
            response = self.check_parse_a_data(get_mock, cancel_type=cancel_type.value)
            assert response.orders[0].cancel_type == cancel_type

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_none_without_cancel_type(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, cancel_type=None)
        assert response.orders[0].cancel_type is None

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_expiry(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, expiry="20190418")
        assert response.orders[0].expiry == datetime(2019, 4, 18).date()

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_none_without_expiry(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, expiry=None)
        assert response.orders[0].expiry is None

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_timestamp(self, get_mock: MagicMock):
        response = self.check_parse_a_data(
            get_mock, timestamp="2024-09-13T15:21:03.059Z"
        )
        assert response.orders[0].timestamp == datetime(
            2024, 9, 13, 15, 21, 3, 59000, tzinfo=timezone.utc
        )

    @patch("gmo_fx.api.api_base.get")
    def test_should_parse_multiple_data(self, get_mock: MagicMock) -> OrdersResponse:
        get_mock.return_value = self.create_response(
            data={"list": [self.create_order_data(), self.create_order_data()]}
        )
        response = self.call_api()
        assert len(response.orders) == 2

    def check_call_with(self, get_mock, **kwargs):
        get_mock.return_value = self.create_response()
        self.call_api(**kwargs)
        kall = get_mock.call_args
        url = kall.args[0] if kall.args else ""
        # Extract query parameters from URL
        if "?" in url:
            query_string = url.split("?", 1)[1]
            params = {}
            for param in query_string.split("&"):
                if "=" in param:
                    key, value = param.split("=", 1)
                    params[key] = value
            return params
        return {}

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_single_root_order_id(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, root_order_id=[123456789])
        assert params["rootOrderId"] == "123456789"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_multiple_root_order_ids(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, root_order_id=[123456789, 987654321])
        assert params["rootOrderId"] == "123456789,987654321"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_empty_root_order_id_list(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, root_order_id=[])
        assert "rootOrderId" not in params

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_single_order_id(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, order_id=[123456789])
        assert params["orderId"] == "123456789"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_multiple_order_ids(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, order_id=[123456789, 223456789])
        assert params["orderId"] == "123456789,223456789"

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_empty_order_id_list(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(get_mock, order_id=[])
        assert "orderId" not in params

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_both_parameters(
        self,
        get_mock: MagicMock,
    ) -> None:
        params = self.check_call_with(
            get_mock, root_order_id=[123456789], order_id=[987654321, 555666777]
        )
        assert params["rootOrderId"] == "123456789"
        assert params["orderId"] == "987654321,555666777"

    @patch("gmo_fx.api.api_base.get")
    def test_should_parse_empty_list(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"list": []})
        response = self.call_api()
        assert len(response.orders) == 0



import hmac
import hashlib
import json
import time

from abc import ABC, abstractmethod
from datetime import datetime
from enum import auto, Enum
from gmo_fx.api.response import Response as ApiResponse
from requests import get, post, Response
from typing import Any, Optional, Type


class ApiBase(ABC):

    class _HttpMethod(Enum):
        GET = "GET"
        POST = "POST"

    VERSION = "v1"

    @property
    @abstractmethod
    def _path(self) -> str:
        pass

    @property
    @abstractmethod
    def _method(self) -> _HttpMethod:
        pass

    @property
    @abstractmethod
    def _endpoint(self) -> str:
        pass

    @property
    def _body(self) -> dict:
        return getattr(self, "_request_body", {})

    def __call__(self, *args: Any, **kwds: Any) -> ApiResponse:

        response: Response = self._call_api(*args, **kwds)
        if response.status_code == 200:
            response_json = response.json()
            return self._response_parser(response_json)

        raise RuntimeError(self._api_error_message(response))

    @property
    def _response_parser(self) -> Type[ApiResponse]:
        pass

    def _api_error_message(self, response: Response):
        return (
            "APIの実行に失敗しました。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def _create_header(
        self,
    ) -> dict:
        return {}

    def _call_api(
        self, path_query: Optional[str] = None, data: Optional[dict] = None
    ) -> Response:
        url = f"{self._endpoint}/{self.VERSION}/{self._path}"
        if path_query:
            url += f"?{path_query}"
        if self._method == self._HttpMethod.GET:
            return get(
                url,
                headers=self._create_header(),
            )
        elif self._method == self._HttpMethod.POST:
            req_body = None
            if data is not None:
                self._request_body = data
                req_body = json.dumps(data)

            return post(url, headers=self._create_header(), data=req_body)
        raise ValueError


class PublicApiBase(ApiBase, ABC):
    @property
    def _endpoint(self) -> str:
        return "https://forex-api.coin.z.com/public"


class PrivateApiBase(ApiBase, ABC):
    def __init__(self, api_key: str, secret_key: str) -> None:
        self._api_key = api_key
        self.__secret_key = secret_key
        super().__init__()

    @property
    def _endpoint(self) -> str:
        return "https://forex-api.coin.z.com/private"

    def _create_header(
        self,
    ) -> dict:
        timestamp = "{0}000".format(int(time.mktime(datetime.now().timetuple())))
        text = timestamp + self._method.value + "/" + self.VERSION + "/" + self._path
        if self._method == self._HttpMethod.POST:
            text += json.dumps(self._body)
        sign = hmac.new(
            bytes(self.__secret_key.encode("ascii")),
            bytes(text.encode("ascii")),
            hashlib.sha256,
        ).hexdigest()
        return {
            "API-KEY": self._api_key,
            "API-TIMESTAMP": timestamp,
            "API-SIGN": sign,
        }



from unittest.mock import MagicMock, patch
from gmo_fx.api.status import StatusApi, StatusResponse, Status

from tests.api_test_base import ApiTestBase


class TestStatusApi(ApiTestBase):

    def call_api(
        self,
    ) -> StatusResponse:
        return StatusApi()()

    @patch("gmo_fx.api.api_base.get")
    def test_status_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_status_open(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"status": "OPEN"})
        actual = self.call_api()
        assert actual.status == Status.OPEN

    @patch("gmo_fx.api.api_base.get")
    def test_status_close(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"status": "CLOSE"})
        actual = self.call_api()
        assert actual.status == Status.CLOSE

    @patch("gmo_fx.api.api_base.get")
    def test_status_maintenance(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data={"status": "MAINTENANCE"})
        actual = self.call_api()
        assert actual.status == Status.MAINTENANCE

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data={"status": "OPEN"})
        self.call_api()
        url = get_mock.mock_calls[0].args[0]
        assert url == "https://forex-api.coin.z.com/public/v1/status"



from datetime import datetime, timedelta, timezone
import re
from typing import Callable, Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.latest_executions import (
    Execution,
    LatestExecutionsApi,
    LatestExecutionsResponse,
)

from tests.api_test_base import ApiTestBase


class TestLatestExecutionsApi(ApiTestBase):

    def create_response(
        self,
        data: Optional[list[dict]] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        return super().create_response(
            data={"list": data},
            status_code=status_code,
            text=text,
        )

    def call_api(
        self,
        symbol: LatestExecutionsApi.Symbol = LatestExecutionsApi.Symbol.USD_JPY,
        count: int = 100,
    ) -> LatestExecutionsResponse:
        return LatestExecutionsApi(
            api_key="",
            secret_key="",
        )(symbol, count)

    def create_execution_data(
        self,
        amount: float = 0.0,
        execution_id: int = 0,
        client_order_id: str = "id",
        order_id: int = 0,
        position_id: int = 0,
        symbol: str = "USD_JPY",
        side: str = "SELL",
        settle_type: str = "CLOSE",
        size: int = 1,
        price: float = 0.0,
        loss_gain: int = 0,
        fee: int = 0,
        settled_swap: float = 0.0,
        timestamp: str = "2022-11-12T13:56:12.02113Z",
    ) -> dict:
        return {
            "amount": str(amount),
            "executionId": execution_id,
            "clientOrderId": client_order_id,
            "orderId": order_id,
            "positionId": position_id,
            "symbol": symbol,
            "side": side,
            "settleType": settle_type,
            "size": str(size),
            "price": str(price),
            "lossGain": str(loss_gain),
            "fee": str(fee),
            "settledSwap": str(settled_swap),
            "timestamp": timestamp,
        }

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    def check_parse_a_data(self, get_mock: MagicMock, **kwargs) -> Execution:
        get_mock.return_value = self.create_response(
            data=[self.create_execution_data(**kwargs)]
        )
        response = self.call_api()
        assert len(response.executions) == 1
        return response.executions[0]

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_amount(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, amount=1.2)
        assert execution.amount == 1.2

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_execution_id(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, execution_id=12)
        assert execution.execution_id == 12

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_client_order_id(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, client_order_id="fdsa")
        assert execution.client_order_id == "fdsa"

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_order_id(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, order_id=12)
        assert execution.order_id == 12

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_id(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, position_id=12)
        assert execution.position_id == 12

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbol(self, get_mock: MagicMock):
        for symbol in Execution.Symbol:
            execution = self.check_parse_a_data(get_mock, symbol=symbol.value)
            assert execution.symbol == symbol

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_side(self, get_mock: MagicMock):
        for side in Execution.Side:
            execution = self.check_parse_a_data(get_mock, side=side.value)
            assert execution.side == side

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settle_type(self, get_mock: MagicMock):
        for settle_type in Execution.SettleType:
            execution = self.check_parse_a_data(get_mock, settle_type=settle_type.value)
            assert execution.settle_type == settle_type

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_size(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, size=1654)
        assert execution.size == 1654

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_price(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, price=100.5)
        assert execution.price == 100.5

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_loss_gain(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, loss_gain=30)
        assert execution.loss_gain == 30

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_fee(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, fee=40)
        assert execution.fee == 40

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_settled_swap(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(get_mock, settled_swap=40.4)
        assert execution.settled_swap == 40.4

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_timestamp(self, get_mock: MagicMock):
        execution = self.check_parse_a_data(
            get_mock, timestamp="2024-01-02T20:11:00.000Z"
        )
        assert execution.timestamp == datetime(2024, 1, 2, 20, 11, tzinfo=timezone.utc)

    @patch("gmo_fx.api.api_base.get")
    def test_should_set_url_symbol(self, get_mock: MagicMock):

        get_mock.return_value = self.create_response(
            data=[self.create_execution_data()]
        )
        response = self.call_api(symbol=LatestExecutionsApi.Symbol.EUR_USD)

        param_match = re.search("\?(.*)", get_mock.mock_calls[0].args[0])
        param = param_match.group(1)
        assert f"symbol=EUR_USD" in param

    @patch("gmo_fx.api.api_base.get")
    def test_should_set_url_count(self, get_mock: MagicMock):

        get_mock.return_value = self.create_response(
            data=[self.create_execution_data()]
        )
        response = self.call_api(count=20)

        param_match = re.search("\?(.*)", get_mock.mock_calls[0].args[0])
        param = param_match.group(1)
        assert f"count=20" in param

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=[])
        self.call_api()
        url_match = re.search("(.*)\?.*", get_mock.mock_calls[0].args[0])
        url = url_match.group(1)
        assert url == "https://forex-api.coin.z.com/private/v1/latestExecutions"
